/**
 * Tests that the analyzeShardKey command returns monotonicity metrics for non-compound shard keys
 * that use hashed sharding.
 *
 * @tags: [requires_fcv_62, featureFlagAnalyzeShardKey]
 */
(function() {
"use strict";

load("jstests/sharding/analyze_shard_key/libs/monotonicity_common.js");

const testCases = [];

// Construct test cases for testing the monotonicity of non-compound shard keys that use hashed
// sharding. For each test case:
// - 'shardKey' is the shard key being analyzed.
// - 'indexKey' is the supporting index for the shard key.
// - 'fieldOpts' specifies the name, type and order for each field inside the documents in the
//   collection. The order refers to whether the value is constant, fluctuating, increasing or
//   decreasing.
// - 'expected' is the expected monotonicity.
for (let orderType0 of kOrderTypes) {
    const fieldName0 = AnalyzeShardKeyUtil.getRandomFieldName("a");
    const fieldType0 = AnalyzeShardKeyUtil.getRandomElement(orderType0.supportedFieldTypes);

    // Test non-compound hashed shard key.
    testCases.push({
        shardKey: {[fieldName0]: "hashed"},
        indexKey: {[fieldName0]: "hashed"},
        fieldOpts: [{name: fieldName0, type: fieldType0, order: orderType0.name}],
        expected: "not monotonic"
    });

    for (let orderType1 of kOrderTypes) {
        const fieldName1 = AnalyzeShardKeyUtil.getRandomFieldName("b");
        const fieldType1 = AnalyzeShardKeyUtil.getRandomElement(orderType1.supportedFieldTypes);

        // Test non-compound hashed shard key.
        testCases.push({
            shardKey: {[fieldName0]: "hashed"},
            indexKey: {[fieldName0]: "hashed", [fieldName1]: 1},
            fieldOpts: [
                {name: fieldName0, type: fieldType0, order: orderType0.name},
                {name: fieldName1, type: fieldType1, order: orderType1.name}
            ],
            expected: "not monotonic"
        });
    }
}

{
    const st = new ShardingTest({shards: 2, rs: {nodes: 2}});

    testAnalyzeShardKeysUnshardedCollection(st.s, testCases);
    testAnalyzeShardKeysShardedCollection(st, testCases);

    st.stop();
}

{
    const rst = new ReplSetTest({nodes: 2});
    rst.startSet();
    rst.initiate();
    const primary = rst.getPrimary();

    testAnalyzeShardKeysUnshardedCollection(primary, testCases);

    rst.stopSet();
}
})();
